/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2001-2002 Vivid Solutions Inc.
 * Copyright (C) 2006 Refractions Research Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation. 
 * See the COPYING file for more information.
 *
 **********************************************************************/

#ifndef GEOS_GEOMUTIL_H
#define GEOS_GEOMUTIL_H

#include <geos/geom/util/GeometryEditor.h>
#include <geos/geom/util/GeometryEditorOperation.h>
#include <geos/geom/util/CoordinateOperation.h>
#include <geos/geom/util/LinearComponentExtracter.h>
#include <geos/geom/util/PointExtracter.h>
#include <geos/geom/util/PolygonExtracter.h>
#include <geos/geom/util/ShortCircuitedGeometryVisitor.h>

namespace geos {
namespace geom { // geos.geom

/** \brief
 * Provides classes that parse and modify Geometry objects.
 */
namespace util { // geos.geom.util

} // namespace geos.geom.util
} // namespace geos.geom
} // namespace geos

#endif
