/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2005-2006 Refractions Research Inc.
 * Copyright (C) 2001-2002 Vivid Solutions Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation. 
 * See the COPYING file for more information.
 *
 **********************************************************************/

#ifndef GEOS_PRECISION_H
#define GEOS_PRECISION_H

namespace geos {

/// Provides classes for manipulating the precision model of Geometries
namespace precision { // geos.precision

} // namespace geos.precision
} // namespace geos

//#include <geos/precision/CommonBits.h>
#include <geos/precision/CommonBitsOp.h>
//#include <geos/precision/CommonBitsRemover.h>
#include <geos/precision/EnhancedPrecisionOp.h>
#include <geos/precision/SimpleGeometryPrecisionReducer.h>

#endif
