/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2006      Refractions Research Inc.
 * Copyright (C) 2001-2002 Vivid Solutions Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation. 
 * See the COPYING file for more information.
 *
 **********************************************************************
 *
 * Kept for backward compatibility, avoid to include
 * directly, include <geos/noding/whatever.h> instead.
 *
 **********************************************************************/

#ifndef GEOS_NODING_H
#define GEOS_NODING_H

//#include <geos/noding/IntersectionAdder.h>
//#include <geos/noding/IntersectionFinderAdder.h>
//#include <geos/noding/IteratedNoder.h>
//#include <geos/noding/MCIndexNoder.h>
//#include <geos/noding/Noder.h>
//#include <geos/noding/NodingValidator.h>
//#include <geos/noding/Octant.h>
//#include <geos/noding/ScaledNoder.h>
//#include <geos/noding/SegmentIntersector.h>
//#include <geos/noding/SegmentNode.h>
//#include <geos/noding/SegmentNodeList.h>
//#include <geos/noding/SegmentString.h>
//#include <geos/noding/SimpleNoder.h>
//#include <geos/noding/SinglePassNoder.h>

namespace geos {

/// Classes to compute nodings for arrangements of line segments and line segment sequences.
namespace noding { // geos.noding

} // namespace geos.noding
} // namespace geos


#endif

