/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2006 Refractions Research Inc.
 * Copyright (C) 2001-2002 Vivid Solutions Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation. 
 * See the COPYING file for more information.
 *
 **********************************************************************/

#ifndef GEOS_INDEXSWEEPLINE_H
#define GEOS_INDEXSWEEPLINE_H

namespace geos {
namespace index { // geos.index

/// \brief
/// Contains classes which implement a sweepline algorithm
/// for scanning geometric data structures.
///
namespace sweepline { // geos.index.sweepline

} // namespace geos.index.sweepline
} // namespace geos.index
} // namespace geos

//#include <geos/index/sweepline/SweepLineEvent.h>
//#include <geos/index/sweepline/SweepLineIndex.h>
//#include <geos/index/sweepline/SweepLineInterval.h>
#include <geos/index/sweepline/SweepLineOverlapAction.h>

#endif
