/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2006 Refractions Research Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation. 
 * See the COPYING file for more information.
 *
 **********************************************************************/

#ifndef GEOS_OPPREDICATE_H
#define GEOS_OPPREDICATE_H

namespace geos {
namespace operation {
namespace predicate {

} // namespace predicate
} // namespace operation
} // namespace geos

//#include <geos/operation/predicate/RectangleContains.h>
//#include <geos/operation/predicate/RectangleIntersects.h>
//#include <geos/operation/predicate/SegmentIntersectionTester.h>

#endif // ifndef GEOS_OPPREDICATE_H
