/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2005-2006 Refractions Research Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation. 
 * See the COPYING file for more information.
 *
 **********************************************************************/

#ifndef GEOS_INLINE_H
#define GEOS_INLINE_H

#ifdef GEOS_INLINE
# define INLINE inline
#else
# define INLINE
#endif


#endif // ndef GEOS_INLINE_H
