/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2001-2002 Vivid Solutions Inc.
 * Copyright (C) 2005 Refractions Research Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation. 
 * See the COPYING file for more information.
 *
 **********************************************************************/

#ifndef GEOS_GEOMGRAPH_INDEX_H
#define GEOS_GEOMGRAPH_INDEX_H

namespace geos {
namespace geomgraph { 
namespace index { 
} 
}
}

#include <geos/geomgraph/index/EdgeSetIntersector.h>
# include <geos/geomgraph/index/SimpleEdgeSetIntersector.h>
# include <geos/geomgraph/index/SimpleSweepLineIntersector.h>
# include <geos/geomgraph/index/SimpleMCSweepLineIntersector.h>

#include <geos/geomgraph/index/SweepLineSegment.h>
#include <geos/geomgraph/index/SweepLineEvent.h>
#include <geos/geomgraph/index/SegmentIntersector.h>
#include <geos/geomgraph/index/MonotoneChain.h>
#include <geos/geomgraph/index/MonotoneChainIndexer.h>
#include <geos/geomgraph/index/MonotoneChainEdge.h>

#endif


