/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2001-2002 Vivid Solutions Inc.
 * Copyright (C) 2006 Refractions Research Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation. 
 * See the COPYING file for more information.
 *
 **********************************************************************/

#ifndef GEOS_NODING_SNAPROUND_H
#define GEOS_NODING_SNAPROUND_H

namespace geos {
namespace noding { // geos.noding

/// \brief
/// Contains classes to implement the Snap Rounding algorithm for
/// noding linestrings.
namespace snapround { // geos.noding.snapround

} // namespace geos.noding.snapround
} // namespace geos.noding
} // namespace geos

//#include <geos/noding/snapround/HotPixel.h>
//#include <geos/noding/snapround/SimpleSnapRounder.h>
//#include <geos/noding/snapround/MCIndexSnapRounder.h>
//#include <geos/noding/snapround/MCIndexPointSnapper.h>

#endif


